<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron" queryBinding="xslt2"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <sch:ns uri="http://www.oxygenxml.com/customFunction" prefix="oxyF"/>
    
    <!-- Ensures the root element contains a prolog section. Displays an error message if the prolog
        with author name and critical dates is missing. -->
    <sch:pattern>
        <sch:rule context="/*">
            <sch:assert test="prolog">
                Should contain prolog with the author name and critical dates.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    
    <!-- Ensures images without @conref or @conkeyref attributes have an alternative text description.
        Issues a warning if the image lacks an "alt" attribute or element. -->
    <sch:pattern id="no_alt_desc">
        <sch:rule context="*[contains(@class, ' topic/image ')][not(@conref)][not(@conkeyref)]">
            <sch:assert test="@alt | alt" 
                flag="non-WAI" role="warning" see="https://www.oxygenxml.com/dita/1.3/specs/langRef/base/image.html">                
                Images must have text alternatives that describe the information or function represented by them.</sch:assert>
        </sch:rule>
    </sch:pattern>   
    
    <!-- Ensures the topic has an ID matching the file name. Displays
        an error message if the topic ID does not equal the file name. -->
    <sch:pattern>
        <sch:rule context="/*[1][contains(@class, ' topic/topic ')]">
            <sch:let name="reqId" value="replace(tokenize(document-uri(/), '/')[last()], '\.dita', '')"/>
            <sch:assert test="@id = $reqId"> Topic ID must be equal to file name. </sch:assert>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern>
        <sch:rule context="*[local-name() = ('topic', 'concept', 'task', 'reference')][title]">
            <sch:assert test="shortdesc" role="warn">
                It is recommended to add an 'shortdesc' in the current.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern>
        <sch:rule context="*[contains(@class, ' topic/xref ') or contains(@class, ' topic/link ')]">
            <sch:report test="@scope = 'external' and @href = text()"> Link text
                is same as @href attribute value. Please remove. </sch:report>
            <!-- Check that the scope attribute is set. -->
            <sch:report test="@format = 'html' and (not(@scope) or @scope != 'external')"
                role="warn"> External link should have a scope attribute set to
                external </sch:report>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern>
        <!-- Image without any kind of reference -->
        <sch:rule context="*[contains(@class, ' topic/image ')]">
            <sch:report test="not(@href) and not(@keyref) and not(@conref) and not(@conkeyref)"> Image without a reference. </sch:report>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern>
        <!-- Report cases when the lines in a codeblock exceeds 95 characters -->
        <sch:rule
            context="*[contains(@class, ' pr-d/codeblock ') and not(@outputclass = 'language-css') and not(@outputclass = 'language-java') and not(@outputclass = 'language-bourne')]"
            role="warn">
            <sch:let name="offendingLines" value="oxyF:lineLengthCheck(string(), 150)"/>
            <sch:report test="string-length($offendingLines) &gt; 0"> Lines (<sch:value-of
                    select="$offendingLines"/>) in codeblocks should not exceed 150 characters.
            </sch:report>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern>
        <!-- Report possible case in which a codeblock containg XML was not marked appropriately. -->
        <sch:rule context="*[contains(@class, ' pr-d/codeblock ')]" role="warn">
            <sch:report test="not(@outputclass)" > Possible XML Codeblock
                without @outputclass set to it. </sch:report>
        </sch:rule>
    </sch:pattern>
    
    <xsl:function name="oxyF:lineLengthCheck" as="xs:string">
        <xsl:param name="textToBeChecked" as="xs:string"/>
        <xsl:param name="maxLength" as="xs:integer"/>
        <xsl:value-of>
            <xsl:for-each select="tokenize($textToBeChecked, '\n')">
                <xsl:if test="string-length(current()) &gt; $maxLength">
                    <xsl:value-of select="concat(position(), ' - ', string-length(current()), ', ')"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:value-of>
    </xsl:function>
</sch:schema>